
VLTLOG_MAPPING = {
  "template" : "vltlog-*",
  "mappings": {
    "fevt": {
      "_all": {
        "enabled": "false"
      },
      "properties": {
        "@timestamp": {
          "type": "date"
        },
        "envname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errlocation": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errseverity": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstack": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstackidx": {
          "type": "long"
        },
        "keywname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "keywvalue": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "loghost": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "logtext": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 256
            }
          }
        },
        "module": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "procid": {
          "type": "long"
        },
        "procname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        }
      }
    },
    "log": {
      "_all": {
        "enabled": "false"
      },
      "properties": {
        "@timestamp": {
          "type": "date"
        },
        "envname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errlocation": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errseverity": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstack": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstackidx": {
          "type": "long"
        },
        "keywname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "keywvalue": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "loghost": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "module": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "procid": {
          "type": "long"
        },
        "procname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        }
      }
    },
    "flog": {
      "_all": {
        "enabled": "false"
      },
      "properties": {
        "@timestamp": {
          "type": "date"
        },
        "envname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errlocation": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errseverity": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstack": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstackidx": {
          "type": "long"
        },
        "keywname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
       },
        "keywvalue": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "loghost": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "module": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "procid": {
          "type": "long"
        },
        "procname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        }
      }
    },
    "err": {
      "_all": {
        "enabled": "false"
      },
      "properties": {
        "@timestamp": {
          "type": "date"
        },
        "envname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errlocation": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errseverity": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstack": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstackidx": {
          "type": "long"
        },
        "keywname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "keywvalue": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "loghost": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "module": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "procid": {
          "type": "long"
        },
        "procname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        }
      }
    },
    "fpar": {
      "_all": {
        "enabled": "false"
      },
      "properties": {
        "@timestamp": {
          "type": "date"
        },
        "envname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errlocation": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errseverity": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstack": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "errstackidx": {
          "type": "long"
        },
        "keywname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "keywvalue": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 100
            }
          }
        },
        "loghost": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "module": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        },
        "procid": {
          "type": "long"
        },
        "procname": {
          "type": "text",
          "fields": {
            "keyword": {
              "type": "keyword",
              "ignore_above": 20
            }
          }
        }
      }
    }
  },
  "settings": {
    "index.number_of_shards" : "4",
    "index.number_of_replicas" : "0",
    "index.query.default_field": "logtext",
    "index.write.wait_for_active_shards": "0"
  },
  "aliases": {
    "vltlog": {}
  }
}

if __name__ == '__main__':
    #ELASTICSEARCH CLIENT PYTHON API BUG put_template doesn't work
    #from elasticsearch import Elasticsearch
    #es = Elasticsearch("http://134.171.189.10")
    #es.put_template(id="vltlog", body=VLTLOG_MAPPING)
    #es.get_template(id="vltlog")

    import requests, json
    url = "http://134.171.189.10:9200/_template/vltlog"
    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    r = requests.post(url, data=json.dumps(VLTLOG_MAPPING), headers=headers)




from datalab.src.importers.es_impoters import es_insert, ES_BASIC_INDEXES, ES_BASIC_INDEXES_FILTERS, ES_BASIC_DOC_TYPES, \
    ES_BASIC_DOC_TYPES_FILTERS, ES_VLTLOG_OPSLOG_GENERATORS_DATA
from datalab.src.importers.kairos_importers import kairos_insert, kairos_parser, kairos_filter
from elasticsearch.helpers import streaming_bulk, parallel_bulk


def insert_datalab(data, kairos_server, es_object, kairos_filter=kairos_filter, kairos_parser=kairos_parser,
                   indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
                   index_suffix=None, doc_types=ES_BASIC_DOC_TYPES,
                   doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS, es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                   bulk_fn=parallel_bulk):
    n_data_inserted = 0;
    response = None
    (kairos_response, n_data_inserted) = kairos_insert(data, kairos_server, kairos_filter, kairos_parser)
    (es_ok, es_result) = es_insert(es_object, data, indexes, indexes_filters, index_suffix, doc_types,
                                   doc_types_filters,
                                   es_generator_data, bulk_fn)
    return (kairos_response, n_data_inserted, es_ok, es_result)





import psycopg2

from elasticsearch import Elasticsearch

from datalab.src.loggers import datalab_loggers

#Alog Connection
DB_SESSION = "dbname=alog host=nico.pl.eso.org user=alogreader"


datalab_logger_object = datalab_loggers.datalab_logger(my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
datalab_logger_connections= datalab_logger_object.datalab_logger

def db_connection_setUp(db_session):
    """
    Creates a new database session

    :param db_session: String database propierties example alogdb: db_session='dbname=alog host=nico.pl.eso.org user=alogreader'
    :type db_session: String
    :return: connection instance

    note:: see http://initd.org/psycopg/docs/module.html
    """
    db_connection = psycopg2.connect(db_session)
    db_connection.set_client_encoding("latin1")
    return db_connection


def db_execute_query(db_connection, query, query_args):
    """
    Execute a query in a database sesion

    :param db_connection: connection db instance
    :param query: String query. example "SELECT * FROM alog WHERE timestamp BETWEEN %s and %s"
    :param query_args: tuple of args of the query. example (time.time(), time.time()-10)
    :type db_connection: Connection
    :type query: String
    :type query_args: tuple
    :return: cursor

    note:: see http://initd.org/psycopg/docs/usage.html
    """
    cursor = db_connection.cursor()
    #datalab_logger_connections.info("reading database[Query. May Take Time]...")
    cursor.execute(query, query_args)
    #datalab_logger_connections.info("finish to query database")
    return cursor


def es_connection_setUp(es_server):
    """
    Establish a elasticsearch connection
    :param es_server:
    :return: connection objetc
    """
    datalab_logger_connections.info("ES : setUp connection")
    return  Elasticsearch(es_server, timeout=30, ignore=[400, 404],  maxsize=100)


import datetime, time

from elasticsearch.helpers import parallel_bulk

from datalab.miscellaneous import SuffixNameFrecuency, _SUFFIX_FRECUENCY_FUNCION_DICT
from datalab.src.connections.connections import db_execute_query
from datalab.src.importers.datalab_importers import insert_datalab
from datalab.src.importers.es_impoters import ES_VLTLOG_OPSLOG_GENERATORS_DATA, ES_BASIC_DOC_TYPES_FILTERS, \
    ES_BASIC_DOC_TYPES, ES_BASIC_INDEXES_FILTERS, ES_BASIC_INDEXES
from datalab.src.importers.kairos_importers import kairos_insert, kairos_parser, kairos_filter, kairos_keep_alive
from datalab.src.loggers import datalab_loggers

datalab_logger_collector_object = datalab_loggers.datalab_logger(my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
datalab_logger_collecter_inserters = datalab_logger_collector_object.datalab_logger

DEBUG = False


def set_debug(boolean):
    global DEBUG
    DEBUG = boolean


def _getcollector_present_piglet(es_object, index, doc_type):
    body = {
        "query": {"match_all": {}},
        "size": 1,
        "sort": [{"collector_piglet": {"order": "desc"}}]
    }
    result = es_object.search(index=index, doc_type=doc_type, body=body)
    timestamp = result['hits']['hits'][0]['_source']['collector_piglet']
    x = datetime.datetime(year=int(timestamp[:4]), month=int(timestamp[5:7]), day=int(timestamp[8:10]),
                          hour=int(timestamp[11:13]), minute=int(timestamp[14:16]), \
                          second=int(timestamp[17:19]), microsecond=int(timestamp[20:]))  # 2017-10-17T23:06:23.165623
    return x


def _getcollector_backwards_piglet(es_object, index, doc_type):
    body = {
        "query": {"match_all": {}},
        "size": 1,
        "sort": [{"collector_piglet": {"order": "asc"}}]
    }
    result = es_object.search(index=index, doc_type=doc_type, body=body)
    timestamp = result['hits']['hits'][0]['_source']['collector_piglet']
    x = datetime.datetime(year=int(timestamp[:4]), month=int(timestamp[5:7]), day=int(timestamp[8:10]),
                          hour=int(timestamp[11:13]), minute=int(timestamp[14:16]), \
                          second=int(timestamp[17:19]), microsecond=int(timestamp[20:]))
    return x


def collector_datalab(db_connection, sql_sentence, sql_args, kairos_server, es_object, kairos_filter=kairos_filter,
                      kairos_parser=kairos_parser,
                      indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
                      index_suffix=None, doc_types=ES_BASIC_DOC_TYPES,
                      doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS, es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                      bulk_fn=parallel_bulk, fetchsize=1000):
    # es = es_connection_setUp([es_server])
    cursor = db_execute_query(db_connection, sql_sentence, sql_args)
    data = cursor.fetchmany(fetchsize)
    kairos_response = "NO_RESPONSE";
    n_data_inserted = 0;
    es_ok = True;
    es_result = 0;
    while data:
        (kairos_response, n_data_inserted, es_ok, es_result) = insert_datalab(data, kairos_server, es_object,
                                                                              kairos_filter,
                                                                              kairos_parser,
                                                                              indexes, indexes_filters,
                                                                              index_suffix, doc_types,
                                                                              doc_types_filters, es_generator_data,
                                                                              bulk_fn)
        data = cursor.fetchmany(fetchsize)
    return (kairos_response, n_data_inserted, es_ok, es_result)


def collector_datalab_period(db_connection, period, kairos_server, es_object, kairos_filter=kairos_filter,
                             kairos_parser=kairos_parser,
                             indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
                             index_suffix=None,
                             doc_types=ES_BASIC_DOC_TYPES,
                             doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
                             es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                             bulk_fn=parallel_bulk, fetchsize=1000):
    return collector_datalab(db_connection, "SELECT * FROM alog WHERE timestamp BETWEEN %s and %s", period,
                             kairos_server, es_object, kairos_filter, kairos_parser, indexes, indexes_filters,
                             index_suffix, doc_types,
                             doc_types_filters, es_generator_data, bulk_fn, fetchsize)


db_logger = datalab_loggers.datalab_logger(name="collector_presentLogger", index_es="collector_present_pigglet",
                                           my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
lastInserted_logger = db_logger.datalab_logger


def collector_datalab_present(db_connection, kairos_server, es_object, delay=100, kairos_filter=kairos_filter,
                              kairos_parser=kairos_parser,
                              indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
                              index_suffix_frecuency=SuffixNameFrecuency.NONE, doc_types=ES_BASIC_DOC_TYPES,
                              doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
                              es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                              bulk_fn=parallel_bulk, fetchsize=1000):
    delay = datetime.timedelta(seconds=delay)
    try:
        past = _getcollector_present_piglet(es_object,"collector_present_pigglet", "python_log")  # datetime.datetime.now();
        present = past + datetime.timedelta(seconds=100)
        while datetime.datetime.utcnow() - present > 0:  # Tenemos retraso! Hay que ponerse al dia e insertar sin delay
            index_suffix= _SUFFIX_FRECUENCY_FUNCION_DICT[index_suffix_frecuency](present)
            (response, n_data_inserted, es_ok, es_result) = collector_datalab_period(db_connection, (past, present),
                                                                                     kairos_server, es_object,
                                                                                     kairos_filter, kairos_parser,
                                                                                     indexes, indexes_filters, index_suffix,
                                                                                     doc_types, doc_types_filters,
                                                                                     es_generator_data, bulk_fn,
                                                                                     fetchsize)
            datalab_logger_collecter_inserters.info(
                "collector_datalab_present : Inserted period: time %s - %s" % (past, present))
            lastInserted_logger.info("collector_datalab_present: LAST_TIMESTAMP_INSERTED: %s %s" % (past, present),
                                     extra={'collector_piglet': present})

            past = present;
            present = past + datetime.timedelta(seconds=100)
        present = datetime.datetime.utcnow()
    except:  # No se ha encontrado al indice, esto quiere decir que nunca se ejecuto el collector
        past = datetime.datetime.utcnow() - datetime.timedelta(seconds=delay.total_seconds())
        present = past + datetime.timedelta(seconds=delay.total_seconds());

    time.sleep(delay.total_seconds())
    while True:
        index_suffix = _SUFFIX_FRECUENCY_FUNCION_DICT[index_suffix_frecuency](present)
        (response, n_data_inserted, es_ok, es_result) = collector_datalab_period(db_connection, (past, present),
                                                                                 kairos_server, es_object,
                                                                                 kairos_filter, kairos_parser,
                                                                                 indexes, indexes_filters, index_suffix,
                                                                                 doc_types, doc_types_filters,
                                                                                 es_generator_data, bulk_fn,
                                                                                 fetchsize)
        datalab_logger_collecter_inserters.info(
            "collector_datalab_present : Inserted period: time %s - %s" % (past, present))
        # datalab_logger_inserted.debug(response.text)
        lastInserted_logger.info("collector_datalab_present: LAST_TIMESTAMP_INSERTED: %s %s" % (past, present),
                                 extra={'collector_piglet': present})
        kairos_keep_alive(kairos_server, 'keepAlive.collector.present')
        past = present;
        datalab_logger_collecter_inserters.info(
            "collector_datalab_present : delay time %s" % str(
                datetime.datetime.utcnow() - past))
        present = datetime.datetime.utcnow()
        if present - past > delay:
            x = (present - past) - delay
            if x > delay:
                time.sleep(0)
            else:
                time.sleep(int((delay - x).total_seconds()))
        else:
            time.sleep(delay.total_seconds())
            # print(past, present)
            # print('2', int(((past + delay - present) + delay).total_seconds()))
            # time.sleep(int(((past + delay - present) + delay).total_seconds()))  # We try to control possible fluction inserting with delay time from alog.
        if DEBUG:  return (response, n_data_inserted, es_ok, es_result)


db2_logger = datalab_loggers.datalab_logger(name="collector_backwardsLogger", index_es="collector_backwards_piglet",
                                            my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
collector_backwardsLogger = db2_logger.datalab_logger


def collector_datalab_backwards(db_connection, kairos_server, es_object, start=-1, kairos_filter=kairos_filter,
                                kairos_parser=kairos_parser,
                                indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS, index_suffix_frecuency=SuffixNameFrecuency.NONE,
                                doc_types=ES_BASIC_DOC_TYPES, doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
                                es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                                bulk_fn=parallel_bulk, fetchsize=1000):
    if start != -1:
        past = start
    else:
        try:
            past = _getcollector_backwards_piglet(es_object,"collector_backwards_piglet", "python_log")  # datetime.datetime.now();
        except:
            past = datetime.datetime.utcnow();
    present = past - datetime.timedelta(seconds=100);
    while True:
        index_suffix = _SUFFIX_FRECUENCY_FUNCION_DICT[index_suffix_frecuency](present)
        result = collector_datalab_period(db_connection, (present, past), kairos_server, es_object, kairos_filter,
                                          kairos_parser, indexes, indexes_filters, index_suffix, doc_types, doc_types_filters,
                                          es_generator_data, bulk_fn, fetchsize)
        datalab_logger_collecter_inserters.info(
            "collector_datalab_backwards : Inserted period: time %s - %s" % (present, past))
        # datalab_logger_inserted.debug(result[0].text)
        collector_backwardsLogger.info("collector_datalab_backwards: LAST_TIMESTAMP_INSERTED: %s %s" % (present, past),
                                       extra={'collector_piglet': present})
        kairos_keep_alive(kairos_server, 'keepAlive.collector.backwards')
        past = present;
        present = present - datetime.timedelta(seconds=100)
        if DEBUG:  return result


def collector_datalab_period_2(db_connection, kairos_server, es_object, period, kairos_filter=kairos_filter,
                               kairos_parser=kairos_parser,
                               indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS, index_suffix_frecuency=SuffixNameFrecuency.NONE,
                               doc_types=ES_BASIC_DOC_TYPES,
                               doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
                               es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA,
                               bulk_fn=parallel_bulk, fetchsize=1000):
    past = period[0]
    while True:
        if period[1] - past > datetime.timedelta(seconds=100):
            present = past + datetime.timedelta(seconds=100)
            datalab_logger_collecter_inserters.info(
                "collector_datalab_period : Inserted period: time %s - %s" % (past, present))
            index_suffix = _SUFFIX_FRECUENCY_FUNCION_DICT[index_suffix_frecuency](present)
            collector_datalab_period(db_connection, (past, present), kairos_server, es_object, kairos_filter,
                                     kairos_parser, indexes, indexes_filters, index_suffix, doc_types, doc_types_filters,
                                     es_generator_data, bulk_fn, fetchsize)
            past = present
        else:
            present = period[1]
            datalab_logger_collecter_inserters.info(
                "collector_datalab_period : Inserted period: time %s - %s" % (past, present))
            index_suffix = _SUFFIX_FRECUENCY_FUNCION_DICT[index_suffix_frecuency](present)
            collector_datalab_period(db_connection, (past, present), kairos_server, es_object, kairos_filter,
                                     kairos_parser, indexes, indexes_filters, index_suffix,doc_types, doc_types_filters,
                                     es_generator_data, bulk_fn, fetchsize)
            datalab_logger_collecter_inserters.info(
                "collector_datalab_period Finish: Terminado de insertar period: time %s - %s" % (period[0], period[1]))
            break;



#TODO Change python2 2 python3

import datetime
import time
import json
from urllib.request import urlopen
from urllib.request import Request
import os
import sys

# ElasticSearch Cluster to Monitor
elasticServer = os.environ.get('ES_METRICS_CLUSTER_URL', 'http://134.171.189.10:9200')
interval = int(os.environ.get('ES_METRICS_INTERVAL', '60'))

# ElasticSearch Cluster to Send Metrics
elasticIndex = os.environ.get('ES_METRICS_INDEX_NAME', 'elasticsearch_metrics')
elasticMonitoringCluster = os.environ.get('ES_METRICS_MONITORING_CLUSTER_URL', 'http://134.171.189.10:9200')


def fetch_clusterhealth():
    try:
        utc_datetime = datetime.datetime.utcnow()
        endpoint = "/_cluster/health"
        urlData = elasticServer + endpoint
        response = urlopen(urlData)
        jsonData = json.loads(response.read())
        clusterName = jsonData['cluster_name']
        jsonData['@timestamp'] = str(utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3])
        if jsonData['status'] == 'green':
            jsonData['status_code'] = 0
        elif jsonData['status'] == 'yellow':
            jsonData['status_code'] = 1
        elif jsonData['status'] == 'red':
            jsonData['status_code'] = 2
        post_data(jsonData)
        return clusterName
    except IOError as err:
        print("IOError: Maybe can't connect to elasticsearch.")
        clusterName = "unknown"
        return clusterName


def fetch_clusterstats():
    utc_datetime = datetime.datetime.utcnow()
    endpoint = "/_cluster/stats"
    urlData = elasticServer + endpoint
    response = urlopen(urlData)
    jsonData = json.loads(response.read())
    jsonData['@timestamp'] = str(utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3]).encode('utf-8')
    post_data(json.dumps(str(jsonData).encode('utf-8')))

def fetch_nodestats(clusterName):
    utc_datetime = datetime.datetime.utcnow()
    endpoint = "/_cat/nodes?v&h=n"
    urlData = elasticServer + endpoint
    response = urlopen(urlData)
    nodes = response.read()[1:-1].strip().decode("utf-8").split('\n')
    for node in nodes:
        endpoint = "/_nodes/%s/stats" % node.rstrip()
        urlData = elasticServer + endpoint
        response = urlopen(urlData)
        jsonData = json.loads(response.read())
        nodeID = jsonData['nodes'].keys()
        try:
            jsonData['nodes'][nodeID[0]]['@timestamp'] = str(utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3])
            jsonData['nodes'][nodeID[0]]['cluster_name'] = clusterName
            newJsonData = jsonData['nodes'][nodeID[0]]
            post_data(newJsonData)
        except:
            continue


def fetch_indexstats(clusterName):
    utc_datetime = datetime.datetime.utcnow()
    endpoint = "/_stats"
    urlData = elasticServer + endpoint
    response = urlopen(urlData)
    jsonData = json.loads(response.read())
    jsonData['_all']['@timestamp'] = str(utc_datetime.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3])
    jsonData['_all']['cluster_name'] = clusterName
    post_data(jsonData['_all'])


def post_data(data):
    utc_datetime = datetime.datetime.utcnow()
    url_parameters = {'cluster': elasticMonitoringCluster, 'index': elasticIndex,
        'index_period': utc_datetime.strftime("%Y.%m.%d"), }
    url = "%(cluster)s/%(index)s-%(index_period)s/message" % url_parameters
    headers = {'content-type': 'application/json'}
    try:
        req = Request(url, headers=headers, data=json.dumps(data))
        f = urlopen(req)
    except Exception as e:
        print("Error:  {}".format(str(e)))


def main():
    clusterName = fetch_clusterhealth()
    if clusterName != "unknown":
        fetch_clusterstats()
        fetch_nodestats(clusterName)
        fetch_indexstats(clusterName)


if __name__ == '__main__':
    try:
        nextRun = 0
        while True:
            if time.time() >= nextRun:
                nextRun = time.time() + interval
                now = time.time()
                main()
                elapsed = time.time() - now
                print("Total Elapsed Time: %s" % elapsed)
                timeDiff = nextRun - time.time()

                # Check timediff , if timediff >=0 sleep, if < 0 send metrics to es
                if timeDiff >= 0:
                    time.sleep(timeDiff)

    except KeyboardInterrupt:
        print('Interrupted')
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)


import unittest
import datetime
import requests, json

from datalab.src.connections.connections import DB_SESSION, db_connection_setUp, es_connection_setUp
from datalab.src.importers.csv_importers import csv_import_file, csv_datalab_import


def _es_is_data_id(es_object, index, doc_type, id):

    result = es_object.search(
        index=index,
        doc_type=doc_type,
        body={
          'query': {
            'terms': {
              '_id': [id],
            }
          }
        }
    )
    return result['hits']['total']==1

def _es_is_data(es_object, index, doc_type, timestamp1, timestamp2):

    result = es_object.search(
        index=index,
        doc_type=doc_type,
        body={
          'query': {
            'range': {
                "@timestamp":{
                    "gte": timestamp1,
                    "lte": timestamp2
                }
            }
          }
        }
    )
    return result['hits']['total']

def _get_number_data_points(server, metric, start, finish):
    query = {
        "metrics": [
        {
            "tags": {},
            "name": metric,
            "aggregators": [
            {
                "name": "sum",
                "align_sampling": True,
                "sampling": {
                    "value": "1",
                    "unit": "milliseconds"
                },
            "align_start_time": False
            }]
        }],
        "cache_time": 0,
        "start_relative": {
        "value": start,
        "unit": "years"
        },
        "end_relative": {
            "value": finish,
            "unit": "years"
        }
    }
    response = requests.post(server + "/api/v1/datapoints/query", data=json.dumps(query))
    return int(response.json()['queries'][0]['sample_size'])

class MyTestCase(unittest.TestCase):
    def test_csv_insert_file(self):
        kairos_server="http://134.171.189.10:8080"
        es_server="http://134.171.189.10"
        path_to_csv = "/Users/user/PycharmProjects/datalab/datalab/tests/wt1tcs.2006-03-10.csv.gz"
        csv_import_file(path_to_csv, kairos_server, es_server, index_suffix="test_csv_insert_file")
        es_object= es_connection_setUp(es_server)
        start=datetime.datetime(2006,1,17,0,0,0)
        finish=datetime.datetime(2006,3,11,1,0,0)
        self.assertTrue(_es_is_data(es_object, index="vltlog", doc_type="log", timestamp1=start, timestamp2=finish))
        self.assertGreater(_get_number_data_points(kairos_server, "TEL.AMBI.TEMP", 12, 10), 0)
        #self.assertTrue(_es_is_data_id(es_object, index="opslog", doc_type="opslog", id="633457081244afecfd05cd229b70a5f5"))
        #self.assertTrue(_es_is_data_id(es_object, index="vltlog", doc_type="err", id="918e07a3993c3bf240765dd2c805fa20"))

    def test_csv_datalab_import(self):
        kairos_server = "http://134.171.189.10:8080"
        es_server = ["134.171.189.10","134.171.189.11","134.171.189.12","134.171.189.13"]
        path_to_csv = "/Users/user/PycharmProjects/datalab/datalab/tests/wt1tcs.2006-0*-1*.csv.gz"
        csv_datalab_import(path_to_csv, kairos_server, es_server, index_suffix="test_csv_datalab_import")
        start=datetime.datetime(2006,1,17,0,0,0)
        finish=datetime.datetime(2006,3,11,1,0,0)
        es_object = es_connection_setUp(es_server)
        self.assertTrue(_es_is_data(es_object, index="vltlog", doc_type="log", timestamp1=start, timestamp2=finish))
        self.assertGreater(_get_number_data_points(kairos_server, "TEL.AMBI.TEMP", 12, 10), 0)
        self.assertGreater(_get_number_data_points(kairos_server, "TEL.ACTO.FOCSIDE", 12, 10), 0)
        #self.assertTrue(_es_is_data_id(es_object, index="opslog", doc_type="opslog", id="633457081244afecf2b529de2347fb3a"))
        #self.assertTrue(_es_is_data_id(es_object, index="vltlog", doc_type="log", id="633457081244afec468020c4dd9c7ef6"))


if __name__ == '__main__':
    unittest.main()


""" JSON serializer for Elasticsearch use
"""
from elasticsearch.serializer import JSONSerializer


class CMRESSerializer(JSONSerializer):
    """ JSON serializer inherited from the elastic search JSON serializer

    Allows to serialize logs for a elasticsearch use.
    Manage the record.exc_info containing an exception type.
    """
    def default(self, data):
        """ Default overrides the elasticsearch default method

        Allows to transform unknown types into strings

        :params data: The data to serialize before sending it to elastic search
        """
        try:
            return super(CMRESSerializer, self).default(data)
        except TypeError:
            return str(data)



import unittest
import datetime
import requests
import json

from datalab.src.importers.kairos_importers import kairos_filter, kairos_parser, kairos_insert

def _get_number_data_points(server, metric):
    query = {
        "metrics": [
        {
            "tags": {},
            "name": metric,
            "aggregators": [
            {
                "name": "sum",
                "align_sampling": True,
                "sampling": {
                    "value": "1",
                    "unit": "milliseconds"
                },
            "align_start_time": False
            }]
        }],
        "cache_time": 0,
        "start_relative": {
        "value": "15",
        "unit": "years"
        }
    }
    response = requests.post(server + "/api/v1/datapoints/query", data=json.dumps(query))
    return int(response.json()['queries'][0]['sample_size'])

class MyTestCase(unittest.TestCase):
    def test_kairos_insert(self):
        kairosdb_server="http://134.171.189.10:8080"
        data_filter=kairos_filter
        data_parser= kairos_parser
        data=[(1546655907,datetime.datetime(2017,9,25,4,11,23),'wat3tcs','lat3vcm','logManager',0,'','FPAR','TESTER_KAIROS_INSERT','2.7182818','Prueba msg','',0,'',''),\
              (1546636293,datetime.datetime(2017,9,25,4,12,27),'wat4tcs','lat4vcm','logManager',0,'','FPAR','TESTER_KAIROS_INSERT','3.1415926','Prueba msg','',0,'','')]
        (response, n_data_inserted) = kairos_insert(data, kairosdb_server, data_filter, data_parser)
        self.assertEqual(response.status_code, 204)
        self.assertEqual(_get_number_data_points("http://134.171.189.10:8080", "TESTER_KAIROS_INSERT"), len(list(filter(data_filter, data))))


if __name__ == '__main__':
    unittest.main()




""" Elasticsearch logging handler
"""

import logging
import datetime
import socket
from threading import Timer, Lock
from enum import Enum
from elasticsearch import helpers as eshelpers
from elasticsearch import Elasticsearch, RequestsHttpConnection

try:
    from requests_kerberos import HTTPKerberosAuth, DISABLED
    CMR_KERBEROS_SUPPORTED = True
except ImportError:
    CMR_KERBEROS_SUPPORTED = False

try:
    from requests_aws4auth import AWS4Auth
    AWS4AUTH_SUPPORTED = True
except ImportError:
    AWS4AUTH_SUPPORTED = False

from cmreslogging.serializers import CMRESSerializer


class CMRESHandler(logging.Handler):
    """ Elasticsearch log handler

    Allows to log to elasticsearch into json format.
    All LogRecord fields are serialised and inserted
    """

    class AuthType(Enum):
        """ Authentication types supported

        The handler supports
         - No authentication
         - Basic authentication
         - Kerberos or SSO authentication (on windows and linux)
        """
        NO_AUTH = 0
        BASIC_AUTH = 1
        KERBEROS_AUTH = 2
        AWS_SIGNED_AUTH = 3

    class IndexNameFrequency(Enum):
        """ Index type supported
        the handler supports
        - Daily indices
        - Weekly indices
        - Monthly indices
        - Year indices
        """
        DAILY = 0
        WEEKLY = 1
        MONTHLY = 2
        YEARLY = 3
        NONE = 4

    # Defaults for the class
    __DEFAULT_ELASTICSEARCH_HOST = [{'host': 'localhost', 'port': 9200}]
    __DEFAULT_AUTH_USER = ''
    __DEFAULT_AUTH_PASSWD = ''
    __DEFAULT_AWS_ACCESS_KEY = ''
    __DEFAULT_AWS_SECRET_KEY = ''
    __DEFAULT_AWS_REGION = ''
    __DEFAULT_USE_SSL = False
    __DEFAULT_VERIFY_SSL = True
    __DEFAULT_AUTH_TYPE = AuthType.NO_AUTH
    __DEFAULT_INDEX_FREQUENCY = IndexNameFrequency.DAILY
    __DEFAULT_BUFFER_SIZE = 1000
    __DEFAULT_FLUSH_FREQ_INSEC = 1
    __DEFAULT_ADDITIONAL_FIELDS = {}
    __DEFAULT_ES_INDEX_NAME = 'python_logger'
    __DEFAULT_ES_DOC_TYPE = 'python_log'
    __DEFAULT_RAISE_ON_EXCEPTION = False
    __DEFAULT_TIMESTAMP_FIELD_NAME = "timestamp"

    __LOGGING_FILTER_FIELDS = ['msecs',
                               'relativeCreated',
                               'levelno',
                               'created']

    @staticmethod
    def _get_daily_index_name(es_index_name):
        """ Returns elasticearch index name
        :param: index_name the prefix to be used in the index
        :return: A srting containing the elasticsearch indexname used which should include the date.
        """
        return "{0!s}-{1!s}".format(es_index_name, datetime.datetime.now().strftime('%Y.%m.%d'))

    @staticmethod
    def _get_weekly_index_name(es_index_name):
        """ Return elasticsearch index name
        :param: index_name the prefix to be used in the index
        :return: A srting containing the elasticsearch indexname used which should include the date and specific week
        """
        current_date = datetime.datetime.now()
        start_of_the_week = current_date - datetime.timedelta(days=current_date.weekday())
        return "{0!s}-{1!s}".format(es_index_name, start_of_the_week.strftime('%Y.%m.%d'))

    @staticmethod
    def _get_monthly_index_name(es_index_name):
        """ Return elasticsearch index name
        :param: index_name the prefix to be used in the index
        :return: A srting containing the elasticsearch indexname used which should include the date and specific moth
        """
        return "{0!s}-{1!s}".format(es_index_name, datetime.datetime.now().strftime('%Y.%m'))

    @staticmethod
    def _get_yearly_index_name(es_index_name):
        """ Return elasticsearch index name
        :param: index_name the prefix to be used in the index
        :return: A srting containing the elasticsearch indexname used which should include the date and specific year
        """
        return "{0!s}-{1!s}".format(es_index_name, datetime.datetime.now().strftime('%Y'))

    @staticmethod
    def _get_only_index_name(es_index_name):
        """ Return elasticsearch index name
        :param: index_name the prefix to be used in the index
        :return: A srting containing the elasticsearch indexname used which should include the date and specific year
        """
        return "{0!s}".format(es_index_name)

    _INDEX_FREQUENCY_FUNCION_DICT = {
        IndexNameFrequency.DAILY: _get_daily_index_name,
        IndexNameFrequency.WEEKLY: _get_weekly_index_name,
        IndexNameFrequency.MONTHLY: _get_monthly_index_name,
        IndexNameFrequency.YEARLY: _get_yearly_index_name,
        IndexNameFrequency.NONE: _get_only_index_name
    }

    def __init__(self,
                 hosts=__DEFAULT_ELASTICSEARCH_HOST,
                 auth_details=(__DEFAULT_AUTH_USER, __DEFAULT_AUTH_PASSWD),
                 aws_access_key=__DEFAULT_AWS_ACCESS_KEY,
                 aws_secret_key=__DEFAULT_AWS_SECRET_KEY,
                 aws_region=__DEFAULT_AWS_REGION,
                 auth_type=__DEFAULT_AUTH_TYPE,
                 use_ssl=__DEFAULT_USE_SSL,
                 verify_ssl=__DEFAULT_VERIFY_SSL,
                 buffer_size=__DEFAULT_BUFFER_SIZE,
                 flush_frequency_in_sec=__DEFAULT_FLUSH_FREQ_INSEC,
                 es_index_name=__DEFAULT_ES_INDEX_NAME,
                 index_name_frequency=__DEFAULT_INDEX_FREQUENCY,
                 es_doc_type=__DEFAULT_ES_DOC_TYPE,
                 es_additional_fields=__DEFAULT_ADDITIONAL_FIELDS,
                 raise_on_indexing_exceptions=__DEFAULT_RAISE_ON_EXCEPTION,
                 default_timestamp_field_name=__DEFAULT_TIMESTAMP_FIELD_NAME):
        """ Handler constructor

        :param hosts: The list of hosts that elasticsearch clients will connect. The list can be provided
                    in the format ```[{'host':'host1','port':9200}, {'host':'host2','port':9200}]``` to
                    make sure the client supports failover of one of the instertion nodes
        :param auth_details: When ```CMRESHandler.AuthType.BASIC_AUTH``` is used this argument must contain
                    a tuple of string with the user and password that will be used to authenticate against
                    the Elasticsearch servers, for example```('User','Password')
        :param aws_access_key: When ```CMRESHandler.AuthType.AWS_SIGNED_AUTH``` is used this argument must contain
                    the AWS key id of the  the AWS IAM user
        :param aws_secret_key: When ```CMRESHandler.AuthType.AWS_SIGNED_AUTH``` is used this argument must contain
                    the AWS secret key of the  the AWS IAM user
        :param aws_region: When ```CMRESHandler.AuthType.AWS_SIGNED_AUTH``` is used this argument must contain
                    the AWS region of the  the AWS Elasticsearch servers, for example```'us-east'
        :param auth_type: The authentication type to be used in the connection ```CMRESHandler.AuthType```
                    Currently, NO_AUTH, BASIC_AUTH, KERBEROS_AUTH are supported
        :param use_ssl: A boolean that defines if the communications should use SSL encrypted communication
        :param verify_ssl: A boolean that defines if the SSL certificates are validated or not
        :param buffer_size: An int, Once this size is reached on the internal buffer results are flushed into ES
        :param flush_frequency_in_sec: A float representing how often and when the buffer will be flushed, even
                    if the buffer_size has not been reached yet
        :param es_index_name: A string with the prefix of the elasticsearch index that will be created. Note a
                    date with YYYY.MM.dd, ```python_logger``` used by default
        :param index_name_frequency: Defines what the date used in the postfix of the name would be. available values
                    are selected from the IndexNameFrequency class (IndexNameFrequency.DAILY,
                    IndexNameFrequency.WEEKLY, IndexNameFrequency.MONTHLY, IndexNameFrequency.YEARLY). By default
                    it uses daily indices.
        :param es_doc_type: A string with the name of the document type that will be used ```python_log``` used
                    by default
        :param es_additional_fields: A dictionary with all the additional fields that you would like to add
                    to the logs, such the application, environment, etc.
        :param raise_on_indexing_exceptions: A boolean, True only for debugging purposes to raise exceptions
                    caused when
        :return: A ready to be used CMRESHandler.
        """
        logging.Handler.__init__(self)

        self.hosts = hosts
        self.auth_details = auth_details
        self.aws_access_key = aws_access_key
        self.aws_secret_key = aws_secret_key
        self.aws_region = aws_region
        self.auth_type = auth_type
        self.use_ssl = use_ssl
        self.verify_certs = verify_ssl
        self.buffer_size = buffer_size
        self.flush_frequency_in_sec = flush_frequency_in_sec
        self.es_index_name = es_index_name
        self.index_name_frequency = index_name_frequency
        self.es_doc_type = es_doc_type
        self.es_additional_fields = es_additional_fields.copy()
        self.es_additional_fields.update({'host': socket.gethostname(),
                                          'host_ip': socket.gethostbyname(socket.gethostname())})
        self.raise_on_indexing_exceptions = raise_on_indexing_exceptions
        self.default_timestamp_field_name = default_timestamp_field_name

        self._client = None
        self._buffer = []
        self._buffer_lock = Lock()
        self._timer = None
        self._index_name_func = CMRESHandler._INDEX_FREQUENCY_FUNCION_DICT[self.index_name_frequency]
        self.serializer = CMRESSerializer()

    def __schedule_flush(self):
        if self._timer is None:
            self._timer = Timer(self.flush_frequency_in_sec, self.flush)
            self._timer.setDaemon(True)
            self._timer.start()

    def __get_es_client(self):
        if self.auth_type == CMRESHandler.AuthType.NO_AUTH:
            if self._client is None:
                self._client = Elasticsearch(hosts=self.hosts,
                                             use_ssl=self.use_ssl,
                                             verify_certs=self.verify_certs,
                                             connection_class=RequestsHttpConnection,
                                             serializer=self.serializer)
            return self._client

        if self.auth_type == CMRESHandler.AuthType.BASIC_AUTH:
            if self._client is None:
                return Elasticsearch(hosts=self.hosts,
                                     http_auth=self.auth_details,
                                     use_ssl=self.use_ssl,
                                     verify_certs=self.verify_certs,
                                     connection_class=RequestsHttpConnection,
                                     serializer=self.serializer)
            return self._client

        if self.auth_type == CMRESHandler.AuthType.KERBEROS_AUTH:
            if not CMR_KERBEROS_SUPPORTED:
                raise EnvironmentError("Kerberos module not available. Please install \"requests-kerberos\"")
            # For kerberos we return a new client each time to make sure the tokens are up to date
            return Elasticsearch(hosts=self.hosts,
                                 use_ssl=self.use_ssl,
                                 verify_certs=self.verify_certs,
                                 connection_class=RequestsHttpConnection,
                                 http_auth=HTTPKerberosAuth(mutual_authentication=DISABLED),
                                 serializer=self.serializer)

        if self.auth_type == CMRESHandler.AuthType.AWS_SIGNED_AUTH:
            if not AWS4AUTH_SUPPORTED:
                raise EnvironmentError("AWS4Auth not available. Please install \"requests-aws4auth\"")
            if self._client is None:
                awsauth = AWS4Auth(self.aws_access_key, self.aws_secret_key, self.aws_region, 'es')
                self._client = Elasticsearch(
                    hosts=self.hosts,
                    http_auth=awsauth,
                    use_ssl=self.use_ssl,
                    verify_certs=True,
                    connection_class=RequestsHttpConnection,
                    serializer=self.serializer
                )
            return self._client

        raise ValueError("Authentication method not supported")

    def test_es_source(self):
        """ Returns True if the handler can ping the Elasticsearch servers

        Can be used to confirm the setup of a handler has been properly done and confirm
        that things like the authentication is working properly

        :return: A boolean, True if the connection against elasticserach host was successful
        """
        return self.__get_es_client().ping()

    @staticmethod
    def __get_es_datetime_str(timestamp):
        """ Returns elasticsearch utc formatted time for an epoch timestamp

        :param timestamp: epoch, including milliseconds
        :return: A string valid for elasticsearch time record
        """
        current_date = datetime.datetime.utcfromtimestamp(timestamp)
        return "{0!s}.{1:03d}Z".format(current_date.strftime('%Y-%m-%dT%H:%M:%S'), int(current_date.microsecond / 1000))

    def flush(self):
        """ Flushes the buffer into ES
        :return: None
        """
        if self._timer is not None and self._timer.is_alive():
            self._timer.cancel()
        self._timer = None

        if self._buffer:
            try:
                with self._buffer_lock:
                    logs_buffer = self._buffer
                    self._buffer = []
                actions = (
                    {
                        '_index': self._index_name_func.__func__(self.es_index_name),
                        '_type': self.es_doc_type,
                        '_source': log_record
                    }
                    for log_record in logs_buffer
                )
                eshelpers.bulk(
                    client=self.__get_es_client(),
                    actions=actions,
                    stats_only=True
                )
            except Exception as exception:
                if self.raise_on_indexing_exceptions:
                    raise exception

    def close(self):
        """ Flushes the buffer and release any outstanding resource

        :return: None
        """
        if self._timer is not None:
            self.flush()
        self._timer = None

    def emit(self, record):
        """ Emit overrides the abstract logging.Handler logRecord emit method

        Format and records the log

        :param record: A class of type ```logging.LogRecord```
        :return: None
        """
        self.format(record)

        rec = self.es_additional_fields.copy()
        for key, value in record.__dict__.items():
            if key not in CMRESHandler.__LOGGING_FILTER_FIELDS:
                rec[key] = "" if value is None else value
        rec[self.default_timestamp_field_name] = self.__get_es_datetime_str(record.created)
        with self._buffer_lock:
            self._buffer.append(rec)

        if len(self._buffer) >= self.buffer_size:
            self.flush()
        else:
            self.__schedule_flush()



import unittest
import datetime, time

import requests, json

from datalab.src.connections.connections import es_connection_setUp
from datalab.src.importers.datalab_importers import insert_datalab
from datalab.src.importers.kairos_importers import kairos_filter

UPDATETIME=2

def _get_number_data_points(server, metric):
    query = {
        "metrics": [
        {
            "tags": {},
            "name": metric,
            "aggregators": [
            {
                "name": "sum",
                "align_sampling": True,
                "sampling": {
                    "value": "1",
                    "unit": "milliseconds"
                },
            "align_start_time": False
            }]
        }],
        "cache_time": 0,
        "start_relative": {
        "value": "15",
        "unit": "years"
        }
    }
    response = requests.post(server + "/api/v1/datapoints/query", data=json.dumps(query))
    return int(response.json()['queries'][0]['sample_size'])

def _es_is_data(es_object, index, doc_type, id):

    result = es_object.search(
        index=index,
        doc_type=doc_type,
        body={
          'query': {
            'terms': {
              '_id': [id],
            }
          }
        }
    )
    return result['hits']['total']==1



class MyTestCase(unittest.TestCase):
    def  test_insert_datalab(self):
        data = [(1546655907, datetime.datetime(2016, 8, 25, 4, 10, 23), 'wat3tcs', 'lat3vcm', 'logManager', 0, '',
                 'FPAR', 'TESTER_DATALAB_INSERT', '4.669201', 'Es Trascendente', '', 0, '', ''),
                (1546636293, datetime.datetime(2016, 8, 25, 4, 12, 27), 'wat4tcs', 'lat4vcm', 'logManager', 0, '',
                 'LOG', 'TESTER_DATALAB_INSERT', 'ESTONOESUNNUMERO', 'La perfeccion.', '', 0, '', '')]
        kairos_server = "http://134.171.189.10:8080"
        es_object = es_connection_setUp("134.171.189.10:9200")
        (kairos_response, n_data_inserted, es_ok, es_result) = insert_datalab(data, kairos_server, es_object, index_suffix="pruebas")
        time.sleep(UPDATETIME)
        self.assertEqual(kairos_response.status_code, 204)
        self.assertEqual(_get_number_data_points("http://134.171.189.10:8080", 'TESTER_DATALAB_INSERT'),
                         len(list(filter(kairos_filter, data))))
        self.assertTrue(_es_is_data(es_object, index="opslog-pruebas", doc_type="opslog", id="9bde7bc7e466ab8dd1fa7f0e9b9482ec"))
        self.assertTrue(_es_is_data(es_object, index="vltlog-pruebas", doc_type="log", id="afa98a536c513056f6e7349c58d22390"))
        es_object.delete(index="opslog-pruebas", doc_type="opslog", id="9bde7bc7e466ab8dd1fa7f0e9b9482ec")
        es_object.delete(index="vltlog-pruebas", doc_type="log", id="afa98a536c513056f6e7349c58d22390")


if __name__ == '__main__':
    unittest.main()



import re, math

import datetime
from enum import Enum

CONNECTION_TIMEOUT_PAUSE = 60

def clean_for_kairos(s):
    """
    Simple kairos parser utility funcion for insert

    :param s: a simple string value to insert in kairos
    :type s: string
    :return: value string parsed ready to insert in kairos
    """
    removelist = r'[^A-Za-z0-9./_-]'
    return re.sub(removelist,"", s)



class SuffixNameFrecuency(Enum):
    """ Index type supported
    the handler supports
    - Daily indices
    - Weekly indices
    - Monthly indices
    - Year indices
    """
    DAILY = 0
    MONTHLY = 2
    YEARLY = 3
    NONE = 5


def _get_daily_suffix_name(date):
    """ Returns elasticearch index name
    :param: index_name the prefix to be used in the index
    :return: A srting containing the elasticsearch indexname used which should include the date.
    """
    return "{0!s}".format(date.strftime('%Y.%m.%d'))



def _get_monthly_suffix_name(date):
    """ Return elasticsearch index name
    :param: index_name the prefix to be used in the index
    :return: A srting containing the elasticsearch indexname used which should include the date and specific moth
    """
    return "{0!s}".format(date.strftime('%Y.%m'))


def _get_yearly_suffix_name(date):
    """ Return elasticsearch index name
    :param: index_name the prefix to be used in the index
    :return: A srting containing the elasticsearch indexname used which should include the date and specific year
    """
    return "{0!s}".format(date.strftime('%Y'))

def _get_none_suffix_name(es_suffix_name):
    """ Return elasticsearch index name
    :param: index_name the prefix to be used in the index
    :return: A srting containing the elasticsearch indexname used which should include the date and specific year
    """
    return None

_SUFFIX_FRECUENCY_FUNCION_DICT = {
    SuffixNameFrecuency.DAILY: _get_daily_suffix_name,
    SuffixNameFrecuency.MONTHLY: _get_monthly_suffix_name,
    SuffixNameFrecuency.YEARLY: _get_yearly_suffix_name,
    SuffixNameFrecuency.NONE: _get_none_suffix_name
}





import json, gzip, requests
import time, datetime

import math

from datalab.miscellaneous import clean_for_kairos
from datalab.src.loggers import datalab_loggers


datalab_logger_kairos_object = datalab_loggers.datalab_logger(my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
datalab_logger_kairos_inserters= datalab_logger_kairos_object.datalab_logger

def kairos_filter(data_point):
    """
    vltlogs and opslog kairos filter. Get only FPAR with numeric keyvalue
    :param data_point:
    :return:
    """
    try:
        x = float(data_point[9])
        if math.isnan(x) or math.isinf(x): return False
        return True
    except:
        return False



def kairos_insert(data, kairosdb_server, kairos_filter, kairos_parser):
    """
    Insert data in kairosdb_server that verify data_filter condition with structure data_parser

    :param data: list of datapoints to insert
    :param kairosdb_server: 'http://kairosdb_server:kairosdb_port'
    :data_filter: funcion of filter of a data point. Return true if point satisfy condicion
    :data_parser: Funcion to parse your data to insert in kairos. see kairos_parser
    :type data: list of list of the data or anything [][] accesible (ex: list of tubles)
    :type kairosdb_server: String
    :type data_filter: data_filter::data_point -> Boolean

    :return: tuple with kairos response object and number of data inserted
    """
    kairosdb_data=[]; n_data_inserted = 0;
    kairosdb_data = list(filter(kairos_filter, data))
    gzipped = gzip.compress(bytes(json.dumps(list(map(kairos_parser, kairosdb_data))), 'latin1'));
    headers = {'content-type': 'application/gzip'}
    response = requests.post(kairosdb_server + "/api/v1/datapoints", gzipped, headers=headers)
    datalab_logger_kairos_inserters.info("KAIROS : Inserted %d data : %d (status code)" % (len(kairosdb_data) if response.status_code == 204 else 0, response.status_code))
    n_data_inserted = len(kairosdb_data) if response.status_code == 204 else 0
    return (response, n_data_inserted)


def kairos_parser(data_point):
    """
    Kairos datalab basic parser

    :param data_point: your data point
    :type data_point: tuple, list etc of values
    :return: dictionary of your datapoint parsed
    :rtype: diccionary
    """
    data_point_parsed = {
        "name": clean_for_kairos(data_point[8]),
        "timestamp":int(time.mktime(data_point[1].timetuple()))*1000, #kairosdb time in miliseconds
        "value"    : float(data_point[9]),
        "tags": {
             "proc": clean_for_kairos(data_point[4]),
             "env": clean_for_kairos(data_point[3]),
             "loghost": clean_for_kairos(data_point[2]),
             "logtext": clean_for_kairos(str(data_point[10]).replace(" ", "_")),
        }
    }
    return data_point_parsed


def kairos_keep_alive(kairosdb_server, metric):
    """
    Utility funcion. Writi in kairos a data with the metric pased and keyvalue 1
    :param kairosdb_server:
    :param metric: name of the metric
    :return:
    """
    keep_alive = [(time.mktime(datetime.datetime.now().timetuple()),datetime.datetime.now(),'datalab','datalab','datalab',0,'','FPAR',metric,'1','Mensaje de keep alive','',0,'','')]
    gzipped = gzip.compress(bytes(json.dumps(list(map(kairos_parser,keep_alive))), 'latin1'))
    headers = {'content-type': 'application/gzip'}
    response = requests.post(kairosdb_server + "/api/v1/datapoints", gzipped, headers=headers)
    return response

if __name__ == "__main__":
    pass;


import multiprocessing
import os, time
import glob

import math
import pandas as pd
import xxhash
import functools

from elasticsearch import ConnectionTimeout
from elasticsearch.helpers import streaming_bulk

from datalab import miscellaneous
from datalab.miscellaneous import clean_for_kairos
from datalab.src.connections.connections import es_connection_setUp
from datalab.src.importers.es_impoters import ES_BASIC_INDEXES, ES_BASIC_INDEXES_FILTERS, ES_BASIC_DOC_TYPES, \
    ES_BASIC_DOC_TYPES_FILTERS, ES_VLTLOG_OPSLOG_GENERATORS_DATA, es_insert_2, _get_index_with_suffix_name, \
    datalab_logger_es_inserters
from datalab.src.importers.kairos_importers import kairos_insert
from datalab.src.loggers import datalab_loggers




datalab_logger_csvImporters_object = datalab_loggers.datalab_logger(my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
datalab_logger_csvInserters = datalab_logger_csvImporters_object.datalab_logger

def csv_opslog_filter(data_point):
    if data_point[0].lower() == 'fpar':
        try:
            x = float(data_point[8])
            if math.isnan(x) or math.isinf(x): return False
            return True
        except:
            return False
    else:
        return False


def csv_kairos_parser(data_point):
    data_point_parsed = {
        "name": clean_for_kairos(data_point[7]),
        "timestamp": int(time.mktime(time.strptime(str(data_point[1]), "%Y-%m-%d %H:%M:%S"))) * 1000,
        # kairosdb time in miliseconds
        "value": float(data_point[8]),
        "tags": {
            "proc": clean_for_kairos(data_point[4]),
            "env": clean_for_kairos(data_point[3]),
            "logh4pysost": clean_for_kairos(data_point[2]),
            "logtext": clean_for_kairos(str(data_point[9]).replace(" ", "_")),
        }
    }
    return data_point_parsed


CSV_VLTLOG_OPSLOG_HEADER = ['typelog', 'tm', 'loghost', 'env', 'procname', 'procid', 'module', 'keyw', 'keyv',
                            'logtext', 'errstack', 'errstackidx', 'errlocation', 'errseverity']


def csv_kairos_import(path_to_csv, kairos_server="http://192.168.1.10:8080", header=CSV_VLTLOG_OPSLOG_HEADER,
                      csv_filter=csv_opslog_filter, kairos_parser=csv_kairos_parser):
    chunksize = 50000
    df = pd.read_csv(filepath_or_buffer=path_to_csv, names=header, compression='gzip', na_values=[""],
                     parse_dates=['tm'], engine='c', encoding='latin-1', usecols=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                     na_filter=False, chunksize=chunksize)
    pool = multiprocessing.Pool(4)
    for chunk in df:
        r = pool.apply_async(kairos_insert,
                             args=[chunk.values, kairos_server, csv_filter, kairos_parser])  # ,callback=callBack)
        # result.append(r.get()) #RECOJER LOS RESULTADOS RALENTIZA X4
    pool.close()
    pool.join()
    return r.get()  # DEVOLVEMOS SOLO EL ULTIMO RESULTADO POR MOTIVOS DE VELOCIDAD. CON ESTE SE PUEDE DEDUCIR SI LA INSERCION FUE BIEN


def csv_es_opslog_generator_data(data,index, doc_type):
    for data_point in data:
        yield {
            '_index': index,
            '_type': doc_type,
            '_id': xxhash.xxh64(str(data_point[10])+str(data_point[11])+str(data_point[12])+str(data_point[13])).hexdigest()+xxhash.xxh64(str(data_point[1])+str(data_point[0])+str(data_point[2])+str(data_point[3])+str(data_point[4])+str(data_point[5])+str(data_point[6])+str(data_point[7])+str(data_point[8])+str(data_point[9])).hexdigest(),
            '_source':  {
                          '@timestamp': data_point[1],
                          'loghost': data_point[2] ,
                          'envname': data_point[3],
                          'procname':data_point[4],
                          'procid':data_point[5],
                          'module':data_point[6],
                          'keywname':data_point[7],
                          'keywvalue': float(data_point[8]),
                          'logtext':data_point[9],
                          'errstack': data_point[10],
                          'errstackidx': data_point[11],
                          'errlocation': data_point[12],
                          'errseverity': data_point[13],
                        },
            }

def csv_es_vltlog_generator_data(data,index, doc_type):
    for data_point in data:
        yield {
            '_index': index,
            '_type': doc_type,
            '_id': xxhash.xxh64(str(data_point[10])+str(data_point[11])+str(data_point[12])+str(data_point[13])).hexdigest()+xxhash.xxh64(str(data_point[1])+str(data_point[0])+str(data_point[2])+str(data_point[3])+str(data_point[4])+str(data_point[5])+str(data_point[6])+str(data_point[7])+str(data_point[8])+str(data_point[9])).hexdigest(),
            '_source':  {
                          '@timestamp': data_point[1],
                          'loghost': data_point[2] ,
                          'envname': data_point[3],
                          'procname':data_point[4],
                          'procid':data_point[5],
                          'module':data_point[6],
                          'keywname':data_point[7],
                          'keywvalue': data_point[8],
                          'logtext':data_point[9],
                          'errstack': data_point[10],
                          'errstackidx': data_point[11],
                          'errlocation': data_point[12],
                          'errseverity': data_point[13],
                        },
            }



CSV_ES_VLTLOG_OPSLOG_GENERATORS_DATA = {
    'vltlog': csv_es_vltlog_generator_data,
    'opslog': csv_es_opslog_generator_data
}

CSV_ES_BASIC_INDEXES = ['vltlog', 'opslog']
CSV_ES_BASIC_INDEXES_FILTERS = {
    'vltlog': lambda data_point: not csv_opslog_filter(data_point),
    'opslog': csv_opslog_filter,
}

CSV_ES_BASIC_DOC_TYPES = {
    'vltlog': ['fpar', 'err', 'flog', 'fevt', 'log'],
    'opslog': ['opslog']
}

CSV_ES_BASIC_DOC_TYPES_FILTERS = {
    'vltlog': {
        'fpar': lambda data_point: data_point[0].lower() == 'fpar',
        'err': lambda data_point: data_point[0].lower() == 'err',
        'flog': lambda data_point: data_point[0].lower() == 'flog',
        'fevt': lambda data_point: data_point[0].lower() == 'fevt',
        'log': lambda data_point: data_point[0].lower() == 'log',
    },
    'opslog': {
        'opslog': lambda data_point: data_point[0].lower() == 'fpar',
        # WE ALREADY INSERTED DATA WHEN I SEE THIS MISTAKE (doc_type should be fpar moraly)
    }
}


def csv_es_import(path_to_csv, es_server, chunksize=50000, header=CSV_VLTLOG_OPSLOG_HEADER,
                  indexes=CSV_ES_BASIC_INDEXES, indexes_filters=CSV_ES_BASIC_INDEXES_FILTERS, index_suffix=None,
                  doc_types=CSV_ES_BASIC_DOC_TYPES, doc_types_filters=CSV_ES_BASIC_DOC_TYPES_FILTERS,
                  es_generator_data=CSV_ES_VLTLOG_OPSLOG_GENERATORS_DATA):
    df = pd.read_csv(filepath_or_buffer=path_to_csv, names=header, compression='gzip', na_values=[""],
                     parse_dates=['tm'], engine='c', encoding='latin-1',
                     na_filter=False,
                     chunksize=chunksize)

    for chunk in df:
        try:
           result = es_insert_2(es_server, chunk.values, indexes, indexes_filters, index_suffix,
                                                     doc_types, doc_types_filters, es_generator_data)  # ,callback=callBack)
        except ConnectionTimeout as timeout: #THE GC is doing hard work
            datalab_logger_csvInserters.error(timeout)
            time.sleep(miscellaneous.CONNECTION_TIMEOUT_PAUSE)
    return result


def csv_import_file(path_to_csv, kairos_server, es_server, es_chunksize=50000,
                    header=CSV_VLTLOG_OPSLOG_HEADER,
                    kairos_filter=csv_opslog_filter, kairos_parser=csv_kairos_parser,
                    indexes=CSV_ES_BASIC_INDEXES, indexes_filters=CSV_ES_BASIC_INDEXES_FILTERS, index_suffix=None, doc_types=CSV_ES_BASIC_DOC_TYPES,
                    doc_types_filters=CSV_ES_BASIC_DOC_TYPES_FILTERS, es_generator_data=CSV_ES_VLTLOG_OPSLOG_GENERATORS_DATA):
    df = pd.read_csv(filepath_or_buffer=path_to_csv, names=header, compression='gzip', na_values=[""],
                     parse_dates=['tm'], engine='c', encoding='latin-1', na_filter=False, chunksize=es_chunksize)

    for chunk in df:
        kairos_result=kairos_insert(chunk.values, kairos_server, kairos_filter, kairos_parser)  # ,callback=callBack)
        try:
            es_result=es_insert_2(es_server, chunk.values, indexes, indexes_filters, index_suffix, doc_types, doc_types_filters, es_generator_data)
        except ConnectionTimeout as timeout: #THE GC is doing hard work
            datalab_logger_csvInserters.error(timeout)
            time.sleep(miscellaneous.CONNECTION_TIMEOUT_PAUSE)
    return (kairos_result, es_result)


def csv_datalab_import(path_to_csv, kairos_server, es_server, es_chunksize=50000,
                       header=CSV_VLTLOG_OPSLOG_HEADER,
                       kairos_filter=csv_opslog_filter, kairos_parser=csv_kairos_parser,
                       indexes=CSV_ES_BASIC_INDEXES, indexes_filters=CSV_ES_BASIC_INDEXES_FILTERS, index_suffix=None,
                       doc_types=CSV_ES_BASIC_DOC_TYPES, doc_types_filters=CSV_ES_BASIC_DOC_TYPES_FILTERS,
                       es_generator_data=CSV_ES_VLTLOG_OPSLOG_GENERATORS_DATA):
    files = glob.glob(path_to_csv)
    total = 0;
    total_inserted = 0;
    bash = 0
    for mi_file in files: total += os.path.getsize(mi_file)
    for f in files:
        s = time.time()
        result = csv_import_file(f, kairos_server, es_server, es_chunksize, header, kairos_filter,
                                 kairos_parser, indexes, indexes_filters, index_suffix,
                                 doc_types, doc_types_filters, es_generator_data)
        total_inserted += os.path.getsize(f);
        bash += os.path.getsize(f)
        if (bash / 1000000) > 20:  # Cada 100MB damos un respiro al sistema
            time.sleep(5)
            bash = 0
        # timeToFinish=int((((total/total_inserted)*(time.time()-s))+s)-time.time())/60
        datalab_logger_csvInserters.info("Inserted: %s file in %.2f secs, %.2f MB-%.2f MB" % (
            f, time.time() - s, total_inserted / 1000000.0, total / 1000000.0))
    return result



import unittest
import requests, json

from datalab.src.importers.samp_importers import samp_import


def _get_number_data_points(server, metric):
    query = {
        "metrics": [
        {
            "tags": {},
            "name": metric,
            "aggregators": [
            {
                "name": "sum",
                "align_sampling": True,
                "sampling": {
                    "value": "1",
                    "unit": "milliseconds"
                },
            "align_start_time": False
            }]
        }],
        "cache_time": 0,
        "start_relative": {
        "value": "15",
        "unit": "years"
        }
    }
    response = requests.post(server + "/api/v1/datapoints/query", data=json.dumps(query))
    return int(response.json()['queries'][0]['sample_size'])

class MyTestCase(unittest.TestCase):
    def test_samp_import(self):
        #path_to_samp="/Users/user/PycharmProjects/datalab/datalab/tests/vtosfSamplingAxis.2017-07-29T220529.ALT.samp"
        path_to_samp="/Users/user/PycharmProjects/datalab/datalab/tests/vtosfSamplingAxis.2017-07-29T220529.ALT.samp"
        kairos_server="http://134.171.189.10:8080"
        samp_import(path_to_samp, kairos_server)
        print(_get_number_data_points(kairos_server, "trackingAxis.vta.alt.POSLOOP.pos"))
        #self.assertEqual(_get_number_data_points(kairos_server, "trackingAxis.vta.az.POSLOOP.pos"),25612)

if __name__ == '__main__':
    unittest.main()



import logging
from datalab.libs.cmreslogging.handlers import CMRESHandler

DATALAB_LOGGER_FORMAT = '%(asctime)s, %(name)-12s, %(levelname)-8s, %(processName)s, %(process)d,  %(pathname)s, %(filename)s, %(module)s, %(funcName)s, %(lineno)d , %(message)s'
class datalab_logger:
    def __init__(self, my_format, name=__name__, index_es="datalab_logger", es_host=[{'host': '134.171.189.10', 'port': 9200}]):  # '%(asctime)s, %(name)-12s, %(levelname)-8s, %(processName)s, %(process)d,  %(pathname)s, %(filename)s, %(module)s, %(funcName)s, %(lineno)d , %(message)s'
        """
        Logger Constructor
        :param my_format: format of log line
        :param name: name of object datalab logger
        :param index_es: index in elascticsearch where to insert logs line generated by this logger
        :param es_host: host of elasticsearch where log lines generated by this logger are going to be inserted
        """
        logging.basicConfig(level=logging.INFO)
        self.datalab_logger = logging.getLogger(name)
        # self.datalab_logger.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter(my_format)
        ch.setFormatter(formatter)
        self.datalab_logger.addHandler(ch)

        handler = CMRESHandler(hosts=es_host,
                               auth_type=CMRESHandler.AuthType.NO_AUTH,
                               index_name_frequency=CMRESHandler.IndexNameFrequency.NONE,
                               es_index_name=index_es) #We Add ES Hander
        self.datalab_logger.addHandler(handler)

    def set_log(self, your_logger):
        """
        Set your logging propierties

        :param your_logger: object logger with your logging fancy propierties
        :type your_logger: logger
        """
        self.datalab_logger = your_logger

if __name__ == "__main__":
    pass;




import curator
import elasticsearch

ONE_WEEK_INDEXES_PREFIX = ['.monitoring-es-6-', '.watcher-history-6' ]
ONE_MONTH_INDEXES_PREFIX = []
ONE_YEAR_INDEXES_PREFIX = []

client = elasticsearch.Elasticsearch("134.171.189.10:9200")

for my_prefix in ONE_WEEK_INDEXES_PREFIX:

    ilo = curator.IndexList(client)

    ilo.filter_by_regex(kind='prefix', value=my_prefix)

    ilo.filter_by_age(source='creation_date', direction='older', unit='days', unit_count=7)

    try :
        curator.DeleteIndices(ilo=ilo, master_timeout=60*60*12).do_action()
    except curator.NoIndices:
        pass



for my_prefix in ONE_MONTH_INDEXES_PREFIX:

    ilo = curator.IndexList(client)

    ilo.filter_by_regex(kind='prefix', value=my_prefix)

    ilo.filter_by_age(source='creation_date', direction='older', unit='months', unit_count=1)

    try:
        curator.DeleteIndices(ilo=ilo, master_timeout=60 * 60 * 12).do_action()
    except curator.NoIndices:
        pass

for my_prefix in ONE_YEAR_INDEXES_PREFIX:

    ilo = curator.IndexList(client)

    ilo.filter_by_regex(kind='prefix', value=my_prefix)

    ilo.filter_by_age(source='creation_date', direction='older', unit='years', unit_count=1)

    try:
        curator.DeleteIndices(ilo=ilo, master_timeout=60 * 60 * 12).do_action()
    except curator.NoIndices:
        pass









import math
from enum import Enum
import datetime

import xxhash as xxhash
from elasticsearch import Elasticsearch
from elasticsearch.helpers import streaming_bulk, parallel_bulk

from datalab.src.connections.connections import es_connection_setUp
from datalab.src.loggers import datalab_loggers

datalab_logger_es_object = datalab_loggers.datalab_logger(my_format=datalab_loggers.DATALAB_LOGGER_FORMAT)
datalab_logger_es_inserters = datalab_logger_es_object.datalab_logger

def es_create_index(es_server, index, mapping):
    es_object = es_connection_setUp(es_server)
    es_object.indices.create(index=index, body=mapping)

def es_vltlogs_generator_data(data, index, doc_type):
    for data_point in data:
        yield {
            '_index': index,
            '_type': doc_type,
            '_id': xxhash.xxh64(str(data_point[10]) + str(data_point[11]) + str(data_point[12]) + str(
                data_point[13])).hexdigest() + xxhash.xxh64(
                str(data_point[1]) + str(data_point[0]) + str(data_point[2]) + str(data_point[3]) + str(
                    data_point[4]) + str(data_point[5]) + str(data_point[6]) + str(data_point[7]) + str(
                    data_point[8]) + str(data_point[9])).hexdigest(),
            '_source': {
                '@timestamp': data_point[1],
                'loghost': data_point[2],
                'envname': data_point[3],
                'procname': data_point[4],
                'procid': data_point[5],
                'module': data_point[6],
                'keywname': data_point[8],
                'keywvalue': data_point[9],
                'logtext': data_point[10],
                'errstack': data_point[11],
                'errstackidx': data_point[12],
                'errlocation': data_point[13],
                'errseverity': data_point[14],
            },
        }


def es_opslogs_generator_data(data, index, doc_type):
    for data_point in data:
        yield {
            '_index': index,
            '_type': doc_type,
            '_id': xxhash.xxh64(str(data_point[10]) + str(data_point[11]) + str(data_point[12]) + str(
                data_point[13])).hexdigest() + xxhash.xxh64(
                str(data_point[1]) + str(data_point[0]) + str(data_point[2]) + str(data_point[3]) + str(
                    data_point[4]) + str(data_point[5]) + str(data_point[6]) + str(data_point[7]) + str(
                    data_point[8]) + str(data_point[9])).hexdigest(),
            '_source': {
                '@timestamp': data_point[1],
                'loghost': data_point[2],
                'envname': data_point[3],
                'procname': data_point[4],
                'procid': data_point[5],
                'module': data_point[6],
                'keywname': data_point[8],
                'keywvalue': float(data_point[9]),
                'logtext': data_point[10],
                'errstack': data_point[11],
                'errstackidx': data_point[12],
                'errlocation': data_point[13],
                'errseverity': data_point[14],
            },
        }


def opslog_filter(data_point):
    if data_point[7].lower() == 'fpar':
        try:
            x = float(data_point[9])
            if math.isnan(x) or math.isinf(x): return False
            return True
        except:
            return False
    else:
        return False


ES_VLTLOG_OPSLOG_GENERATORS_DATA = {
    'vltlog': es_vltlogs_generator_data,
    'opslog': es_opslogs_generator_data
}

ES_BASIC_INDEXES = ['vltlog', 'opslog']
ES_BASIC_INDEXES_FILTERS = {
    'vltlog': lambda data_point: not opslog_filter(data_point),
    'opslog': opslog_filter,
}

ES_BASIC_DOC_TYPES = {
    'vltlog': ['fpar', 'err', 'flog', 'fevt', 'log'],
    'opslog': ['opslog']
}

ES_BASIC_DOC_TYPES_FILTERS = {
    'vltlog': {
        'fpar': lambda data_point: data_point[7].lower() == 'fpar',
        'err': lambda data_point: data_point[7].lower() == 'err',
        'flog': lambda data_point: data_point[7].lower() == 'flog',
        'fevt': lambda data_point: data_point[7].lower() == 'fevt',
        'log': lambda data_point: data_point[7].lower() == 'log',
    },
    'opslog': {
        'opslog': lambda data_point: data_point[7].lower() == 'fpar',
        # WE ALREADY INSERTED DATA WHEN I SEE THIS MISTAKE (doc_type should be fpar moraly)
    }
}


def _get_index_with_suffix_name(es_index_name, suffix):
    """ Return elasticsearch index name
    :param: index_name the prefix to be used in the index
    :return: A srting containing the elasticsearch indexname used which should include the suffix
    """
    if suffix:
        return "{0!s}-{1!s}".format(es_index_name, suffix)
    else:
        return es_index_name


def es_insert(es_object, data, indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
              index_suffix=None,
              doc_types=ES_BASIC_DOC_TYPES, doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
              es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA, bulk_fn=streaming_bulk):
    ok = True;
    result = "NO_INSERTED"
    datalab_logger_es_inserters.info("ES : Inserting data")
    for index in indexes:
        for doc_type in doc_types[index]:
            for ok, result in bulk_fn(es_object, es_generator_data[index](
                    list(filter(doc_types_filters[index][doc_type], list(filter(indexes_filters[index], data)))),
                    _get_index_with_suffix_name(index, index_suffix),
                    doc_type)): pass;
    datalab_logger_es_inserters.info(
        "ES : Finish Insert Chunk : Ok? %s" % ok)  # WE DON'T COLLECT RESULTS FOR PERFORMENCES REASONS

    return (ok, result)


def es_insert_2(es_server, data, indexes=ES_BASIC_INDEXES, indexes_filters=ES_BASIC_INDEXES_FILTERS,
                index_suffix=None,
                doc_types=ES_BASIC_DOC_TYPES, doc_types_filters=ES_BASIC_DOC_TYPES_FILTERS,
                es_generator_data=ES_VLTLOG_OPSLOG_GENERATORS_DATA, bulk_fn=streaming_bulk):
    ok = True;
    result = "NO_INSERTED"
    es_object = es_connection_setUp(es_server)
    datalab_logger_es_inserters.info("ES : Inserting data")

    for index in indexes:
        for doc_type in doc_types[index]:
            for ok, result in bulk_fn(es_object, es_generator_data[index](
                    list(filter(doc_types_filters[index][doc_type], list(filter(indexes_filters[index], data)))),
                    _get_index_with_suffix_name(index, index_suffix),
                    doc_type)): pass;
    datalab_logger_es_inserters.info(
        "ES : Finish Insert Chunk : Ok? %s" % ok)  # WE DON'T COLLECT RESULTS FOR PERFORMENCES REASONS
    return (ok, result)




OPSLOG_MAPPING = {
    "template" : "opslog-*",
    "mappings": {
        "opslog": {
            "_all": {
                "enabled": "false"
            },
            "properties": {
                "@timestamp": {
                    "type": "date"
                },
                "envname": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "errlocation": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "errseverity": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "errstack": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "errstackidx": {
                    "type": "long"
                },
                "keywname": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 100
                        }
                    }
                },
                "keywvalue": {
                    "type": "double"
                },
                "loghost": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "logtext": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 256
                        }
                    }
                },
                "module": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                },
                "procid": {
                    "type": "long"
                },
                "procname": {
                    "type": "text",
                    "fields": {
                        "keyword": {
                            "type": "keyword",
                            "ignore_above": 20
                        }
                    }
                }
            }
        }
    },
    "settings": {
        "index.number_of_shards": "4",
        "index.number_of_replicas": "0",
        "index.query.default_field": "logtext",
        "index.write.wait_for_active_shards": "0"
    },
    "aliases": {
        "opslog": {}
    }
}


if __name__ == '__main__':
    # ELASTICSEARCH CLIENT PYTHON API BUG put_template doesn't work
    # from elasticsearch import Elasticsearch
    # es = Elasticsearch("http://134.171.189.10")
    # es.put_template(id="opslog", body=OPSLOG_MAPPING)
    # es.get_template(id="opslog")

    import requests, json

    #Delete de template
    url = "http://134.171.189.10:9200/_template/opslog"
    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    r = requests.delete(url, headers=headers)

    #Create the new template
    url = "http://134.171.189.10:9200/_template/opslog"
    headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
    r = requests.post(url, data=json.dumps(OPSLOG_MAPPING), headers=headers)









import datetime, time
import math

import functools
import requests, json, gzip
import linecache

import pandas as pd



def _samp_kairos_parser(keyname, tags, tsp_kv):
    """

    :param keyname:
    :param tags:
    :param tsp_kv:
    :return:
    """
    t=datetime.datetime.strptime(str(tsp_kv[0])+str(tsp_kv[1]),"%Y-%m-%d%H:%M:%S.%f")
    data_point_parsed = {
        "name": keyname,
        "timestamp": int(time.mktime(t.timetuple()))*1000 + int(t.microsecond/1000), #kairosdb time in miliseconds
        "value"    : float(tsp_kv[2]),
        "tags": tags
    }
    return data_point_parsed


def _samp_kairos_clean(x):
    """

    :param x:
    :return:
    """
    try:
        y = float(x[2])
        if math.isnan(y) or math.isinf(y):
            return False
        datetime.datetime.strptime(str(x[0])+str(x[1]), "%Y-%m-%d%H:%M:%S.%f")
        return True
    except:
        return False

def _samp_kairos_insert(kairosdb_server, keyname, tsp_kv, tags):
    """

    :param kairosdb_server:
    :param keyname:
    :param tsp_kv:
    :param tags:
    :return:
    """
    kairosdb_data=[]; n_data_inserted = 0;
    kairosdb_data = list(filter(_samp_kairos_clean, tsp_kv))
    f = functools.partial(_samp_kairos_parser, keyname, tags)
    gzipped = gzip.compress(bytes(json.dumps(list(map(f, kairosdb_data))), 'latin1'));
    headers = {'content-type': 'application/gzip'}
    response = requests.post(kairosdb_server + "/api/v1/datapoints", gzipped, headers=headers)
    #datalab_logger_inserted.info("KAIROS : Inserted %d data : %d (status code)" % response.status_code)
    n_data_inserted = len(kairosdb_data) if response.status_code == 204 else 0
    #datalab_logger_inserted.info("KAIROS : Error inserting data")
    return (response, n_data_inserted)

def samp_import(path_to_samp,kairos_server):
    """

    :param path_to_samp:
    :param kairos_server:
    :return:
    """
    chunksize=10000
    env = linecache.getline(path_to_samp, 3).split(' ')[2][1:-1].replace(":", "")
    tags = {'env':env}
    l = linecache.getline(path_to_samp, 5)
    ls = l.split(' ');
    attributes=int(ls[4]) #N of atributes
    keyvnames = ['Record Index', 'Time Sequence', 'yyyy-mm-dd', 'hh:mm:ss.uuuu']
    for i in range(0,attributes): # We get the keyvnames
        l = linecache.getline(path_to_samp, 14+i)
        keyvname = (" ".join(l.split()).split())[2].replace(":", ".").replace("\"", "")
        if keyvname[0] == '.': keyvname=keyvname[1:]
        keyvnames.append(keyvname)
    headers_rows=range(1,13+attributes+9)
    df = pd.read_csv(filepath_or_buffer=path_to_samp, names=keyvnames, na_values=[""], skiprows=headers_rows, delim_whitespace=True, chunksize=chunksize)
    for chunk in df:
        for i in range(1, attributes + 1):
            kairos_result=_samp_kairos_insert(kairos_server, keyvnames[3+i], chunk[['yyyy-mm-dd', 'hh:mm:ss.uuuu', keyvnames[3+i]]].values, tags) #We Insert Timestamp-Keyvalues
    if "#___oOo___" in str(chunk[['Record Index']][-1:]):
        print("End of File reach")
    else:
        print("Onlyne file")



def _g(x,y):
    return x<y

UPDATETIME=100
def samp_online_import(path_to_samp,kairos_server):
    """

    :param path_to_samp:
    :param kairos_server:
    :return:
    """
    chunksize=10000
    env = linecache.getline(path_to_samp, 3).split(' ')[2][1:-1].replace(":", "")
    tags = {'env':env}
    l = linecache.getline(path_to_samp, 5)
    ls = l.split(' ');
    attributes=int(ls[4]) #N of atributes
    keyvnames = ['Record Index', 'Time Sequence', 'yyyy-mm-dd', 'hh:mm:ss.uuuu']
    for i in range(0,attributes): # We get the keyvnames
        l = linecache.getline(path_to_samp, 14+i)
        keyvname = (" ".join(l.split()).split())[2].replace(":", ".").replace("\"", "")
        if keyvname[0] == '.': keyvname=keyvname[1:]
        keyvnames.append(keyvname)
    header_lines=13+attributes+9
    f = functools.partial(_g, x=header_lines)
    while True:
        df = pd.read_csv(filepath_or_buffer=path_to_samp, names=keyvnames, na_values=[""], skiprows=f, delim_whitespace=True, chunksize=chunksize,)
        for chunk in df:
            for i in range(1, attributes + 1):
                kairos_result=_samp_kairos_insert(kairos_server, keyvnames[3+i], chunk[['yyyy-mm-dd', 'hh:mm:ss.uuuu', keyvnames[3+i]]].values, tags) #We Insert Timestamp-Keyvalues
        if "#___oOo___" in str(chunk[['Record Index']][-1:]):
            print("End of File reach")
            break
        else:
            print("Onlyne file")
            x=int(chunk[['Record Index']].iloc[-1])
            f = functools.partial(_g,x=header_lines+x)
            time.sleep(UPDATETIME)

if __name__ == "__main__":
    pass;





import unittest
import datetime, time

from datalab.src.connections.connections import es_connection_setUp
from datalab.src.importers.es_impoters import es_insert, es_insert_2

UPDATETIME=2

def _es_is_data(es_object, index, doc_type, id):

    result = es_object.search(
        index=index,
        doc_type=doc_type,
        body={
          'query': {
            'terms': {
              '_id': [id],
            }
          }
        }
    )
    return result['hits']['total']==1



class MyTestCase(unittest.TestCase):
    def test_es_insert(self):
        data = [(1546655907,datetime.datetime(2017,9,25,4,11,23),'wat3tcs','lat3vcm','logManager',0,'','FPAR','TESTER_ES_INSERT','4.6692','Es Trascendente','',0,'',''),\
                (1546636293,datetime.datetime(2017,9,25,4,12,27),'wat4tcs','lat4vcm','logManager',0,'','FPAR','TESTER_ES_INSERT','1.6180','La perfeccion.','',0,'',''),
                (1546616293, datetime.datetime(2017, 9, 25, 4, 12, 27), 'wat4tcs', 'lat4vcm', 'logManager', 0, '', 'FPAR', 'TESTER_ES_INSERT', 'ESTONOESUNNUMERO', 'La perfeccion.', '', 0, '', ''),
                (1546616293, datetime.datetime(2017, 9, 25, 4, 12, 27), 'wat4tcs', 'lat4vcm', 'logManager', 0, '','LOG', 'TESTER_ES_INSERT', 'ESTONOESUNNUMERO', 'La perfeccion.', '', 0, '', '')]
        es = es_connection_setUp("134.171.189.10:9200")
        (ok, result) = es_insert(es_object=es, data=data, index_suffix="test_es_insert".lower())
        time.sleep(UPDATETIME)
        self.assertEqual(ok, True)
        self.assertTrue(_es_is_data(es, index="opslog-test_es_insert", doc_type="opslog", id="afa98a536c513056ac60368c222bda4c"))
        self.assertTrue(_es_is_data(es, index="opslog", doc_type="opslog", id="9bde7bc7e466ab8db180492c96020d5b"))
        self.assertTrue(_es_is_data(es, index="vltlog-test_es_insert", doc_type="fpar", id="afa98a536c513056689fa883f14f831a"))
        self.assertTrue(_es_is_data(es, index="vltlog", doc_type="log", id="afa98a536c513056ae1269ccf4fd8807"))
        es.delete(index="opslog-test_es_insert", doc_type="opslog", id="afa98a536c513056ac60368c222bda4c")
        es.delete(index="opslog-test_es_insert", doc_type="opslog", id="9bde7bc7e466ab8db180492c96020d5b")
        es.delete(index="vltlog-test_es_insert", doc_type="fpar", id="afa98a536c513056689fa883f14f831a")
        es.delete(index="vltlog-test_es_insert", doc_type="log", id="afa98a536c513056ae1269ccf4fd8807")

    def test_es_insert_2(self):
        data = [(1546655907,datetime.datetime(2017,9,25,4,11,23),'wat3tcs','lat3vcm','logManager',0,'','FPAR','TESTER_ES_INSERT','4.6692','Es Trascendente','',0,'',''),\
                (1546636293,datetime.datetime(2017,9,25,4,12,27),'wat4tcs','lat4vcm','logManager',0,'','FPAR','TESTER_ES_INSERT','1.6180','La perfeccion.','',0,'',''),
                (1546616293, datetime.datetime(2017, 9, 25, 4, 12, 27), 'wat4tcs', 'lat4vcm', 'logManager', 0, '', 'FPAR', 'TESTER_ES_INSERT', 'ESTONOESUNNUMERO', 'La perfeccion.', '', 0, '', ''),
                (1546616293, datetime.datetime(2017, 9, 25, 4, 12, 27), 'wat4tcs', 'lat4vcm', 'logManager', 0, '','LOG', 'TESTER_ES_INSERT', 'ESTONOESUNNUMERO', 'La perfeccion.', '', 0, '', '')]
        (ok, result) = es_insert_2(es_server="134.171.189.10:9200", data=data, index_suffix="test_es_insert_2".lower())
        time.sleep(UPDATETIME)
        self.assertEqual(ok, True)
        es = es_connection_setUp("134.171.189.10:9200")
        self.assertTrue(_es_is_data(es, index="opslog-test_es_insert_2", doc_type="opslog", id="afa98a536c513056ac60368c222bda4c"))
        self.assertTrue(_es_is_data(es, index="opslog", doc_type="opslog", id="9bde7bc7e466ab8db180492c96020d5b"))
        self.assertTrue(_es_is_data(es, index="vltlog-test_es_insert_2", doc_type="fpar", id="afa98a536c513056689fa883f14f831a"))
        self.assertTrue(_es_is_data(es, index="vltlog", doc_type="log", id="afa98a536c513056ae1269ccf4fd8807"))
        es.delete(index="opslog-test_es_insert_2", doc_type="opslog", id="afa98a536c513056ac60368c222bda4c")
        es.delete(index="opslog-test_es_insert_2", doc_type="opslog", id="9bde7bc7e466ab8db180492c96020d5b")
        es.delete(index="vltlog-test_es_insert_2", doc_type="fpar", id="afa98a536c513056689fa883f14f831a")
        es.delete(index="vltlog-test_es_insert_2", doc_type="log", id="afa98a536c513056ae1269ccf4fd8807")



if __name__ == '__main__':
    unittest.main()



import requests
import datetime, time
import pandas as pd
import json

def convertHumanTime2kairosTime(s):
    t=datetime.datetime.strptime(str(s),"%Y-%m-%d %H:%M:%S.%f")
    return int(time.mktime(t.timetuple()))*1000 + int(t.microsecond/1000)

def _getkw(server, old, new, metric):
    query = {
        "metrics": [
        {
            "tags": {},
            "name": metric,
            "aggregators": [
            {
                "name": "sum",
                "align_sampling": True,
                "sampling": {
                    "value": "1",
                    "unit": "milliseconds"
                },
            "align_start_time": False
            }]
        }],
        "cache_time": 0,
        "start_absolute": convertHumanTime2kairosTime(old),
        "end_absolute": convertHumanTime2kairosTime(new)
     }
    response = requests.post(server + "/api/v1/datapoints/query", data=json.dumps(query))
    return response.json()["queries"][0]['results'][0]['values']


def get_kairos_data(server, old, new, metric):
    return [x[1] for x in _getkw(server, old, new, metric)]

def get_kairos_timestamp(server, old, new, metric):
    return [x[0] for x in _getkw(server, old, new, metric)]


server = "http://134.171.189.10:8080"
metric = "INS.AO.SENS1.VAL1"

from_ = "2017-08-01 22:22:22.003"
to = "2017-08-01 22:25:22.003"

def getkw(server, old, new, metrics):
    merge = pd.Series(get_kairos_data(server, old, new, metrics[0]), index=pd.to_datetime(get_kairos_timestamp(server, old, new, metrics[0]), unit='ms'))
    for metric in metrics[1:]:
        serie = pd.Series(get_kairos_data(server, old, new, metric), index=pd.to_datetime(get_kairos_timestamp(server, old, new, metric), unit='ms'))
        merge = pd.concat([merge, serie], axis=1)
    return merge.resample('10s').mean().interpolate(method='nearest').fillna(method='pad').fillna(method='bfill')







import unittest
import datetime, time

from datalab.miscellaneous import SuffixNameFrecuency
from datalab.src.connections.connections import DB_SESSION, es_connection_setUp, db_connection_setUp
from datalab.src.importers import collectors_importers
from datalab.src.importers.collectors_importers import collector_datalab_period, collector_datalab_present, \
    collector_datalab_backwards, collector_datalab_period_2

UPDATETIME=2


def _es_is_data(es_object, index, doc_type, timestamp1, timestamp2):

    result = es_object.search(
        index=index,
        doc_type=doc_type,
        body={
          'query': {
            'range': {
                "@timestamp":{
                    "gte": timestamp1,
                    "lte": timestamp2
                }
            }
          }
        }
    )
    return result['hits']['total']

class MyTestCase(unittest.TestCase):
    def test_collector_datalab_period(self):
        db_connection = db_connection_setUp(DB_SESSION)
        kairos_server = "http://134.171.189.10:8080"
        es_object = es_connection_setUp("134.171.189.10:9200")
        present = datetime.datetime.utcnow()
        past= present - datetime.timedelta(seconds=10)
        period = (past, present)
        collector_datalab_period(db_connection, period, kairos_server, es_object, index_suffix="test_collector_datalab_period")
        time.sleep(UPDATETIME)
        self.assertGreater(_es_is_data(es_object, 'vltlog', 'fpar', past, present), 0)

    def test_collector_datalab_present(self):
        collectors_importers.set_debug(True)
        db_connection = db_connection_setUp(DB_SESSION)
        kairos_server = "http://134.171.189.10:8080"
        es_object = es_connection_setUp("134.171.189.10:9200")
        delay=100
        past = datetime.datetime.utcnow() - datetime.timedelta(seconds=delay)
        present = past + datetime.timedelta(seconds=100)
        collector_datalab_present(db_connection, kairos_server, es_object, delay=delay, index_suffix_frecuency=SuffixNameFrecuency.MONTHLY)
        time.sleep(UPDATETIME)
        self.assertGreater(_es_is_data(es_object, 'vltlog', 'fpar', past, present), 0)

    def test_collector_datalab_backwards(self):
        collectors_importers.set_debug(True)
        db_connection = db_connection_setUp(DB_SESSION)
        kairos_server = "http://134.171.189.10:8080"
        es_object = es_connection_setUp("134.171.189.10:9200")
        start=datetime.datetime(2017,9,25,4,11,23)
        past=start-datetime.timedelta(seconds=10)
        collector_datalab_backwards(db_connection, kairos_server, es_object, start=start, index_suffix_frecuency=SuffixNameFrecuency.DAILY)
        time.sleep(UPDATETIME)
        self.assertGreater(_es_is_data(es_object, 'vltlog', 'fpar', past, start), 0)

    def test_datalab_period_2(self):
        db_connection = db_connection_setUp(DB_SESSION)
        kairos_server = "http://134.171.189.10:8080"
        es_object = es_connection_setUp("134.171.189.10:9200")
        start=datetime.datetime(2016,10,25,4,11,23)
        past=start-datetime.timedelta(seconds=100)
        period=(past,start)
        collector_datalab_period_2(db_connection, kairos_server, es_object, period, index_suffix_frecuency=SuffixNameFrecuency.DAILY)
        time.sleep(UPDATETIME)
        self.assertGreater(_es_is_data(es_object, 'vltlog', 'fpar', past, start), 0)

if __name__ == '__main__':
    unittest.main()
